%define anolis_release 1
%global toolchain clang

%undefine _include_frame_pointers

%global maj_ver 16
%global libomp_version %{maj_ver}.0.6
%global libomp_srcdir openmp-%{libomp_version}.src
%global cmake_srcdir cmake-%{libomp_version}.src

%global libomp_arch %{_arch}

Name: libomp
Version: %{libomp_version}
Release: %{anolis_release}%{?dist}
Summary: OpenMP runtime for clang

License: Apache-2.0 WITH LLVM-exception OR NCSA
URL: http://openmp.llvm.org
Source0: https://github.com/llvm/llvm-project/releases/download/llvmorg-%{libomp_version}/%{libomp_srcdir}.tar.xz
Source3: run-lit-tests
Source4: lit.distro.cfg.py
Source5: https://github.com/llvm/llvm-project/releases/download/llvmorg-%{libomp_version}/%{cmake_srcdir}.tar.xz

BuildRequires: clang
# For clang-offload-packager
BuildRequires: clang-tools-extra
BuildRequires: cmake
BuildRequires: ninja-build
BuildRequires: elfutils-libelf-devel
BuildRequires: perl
BuildRequires: perl-Data-Dumper
BuildRequires: perl-Encode
BuildRequires: perl-FindBin
BuildRequires: libffi-devel
BuildRequires: libxml2-devel

# libomptarget needs the llvm cmake files
BuildRequires: llvm-devel

Requires: elfutils-libelf

%description
OpenMP runtime for clang.

%package devel
Summary: OpenMP header files
Requires: %{name} = %{version}-%{release}
Requires: clang-resource-filesystem = %{version}

%description devel
OpenMP header files.

%package test
Summary: OpenMP regression tests
Requires: %{name} = %{version}-%{release}
Requires: %{name}-devel = %{version}-%{release}
Requires: clang
Requires: llvm
Requires: python3-lit

%description test
OpenMP regression tests

%prep
%setup -T -q -b 5 -n %{cmake_srcdir}
cd ..
mv %{cmake_srcdir} cmake
%autosetup -n %{libomp_srcdir} -p2

%build

%cmake	-GNinja \
	-DLIBOMP_INSTALL_ALIASES=OFF \
	-DCMAKE_MODULE_PATH=%{_libdir}/cmake/llvm \
	-DLLVM_DIR=%{_libdir}/cmake/llvm \
	-DCMAKE_INSTALL_INCLUDEDIR=%{_libdir}/clang/%{maj_ver}/include \
	-DOPENMP_LIBDIR_SUFFIX=64 \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
        -DLIBOMP_HAVE_VERSION_SCRIPT_FLAG:BOOL=ON

%cmake_build


%install
%cmake_install

# Test package setup
%global libomp_srcdir %{_datadir}/libomp/src/
%global libomp_testdir %{libomp_srcdir}/runtime/test/
%global lit_cfg %{libomp_testdir}/%{_arch}.site.cfg.py
%global lit_distro_cfg %{_datadir}/libomp/lit.distro.cfg.py

install -d %{buildroot}%{libomp_srcdir}/runtime
cp -R runtime/test  %{buildroot}%{libomp_srcdir}/runtime
cp -R runtime/src  %{buildroot}%{libomp_srcdir}/runtime

# Generate lit config files.  Strip off the last line that initiates the
# test run, so we can customize the configuration.
head -n -1 %{_vpath_builddir}/runtime/test/lit.site.cfg >> %{buildroot}%{lit_cfg}

# Install custom distro config file
cp %{SOURCE4} %{buildroot}%{lit_distro_cfg}

# Patch lit config files to load custom distro config
echo "lit_config.load_config(config, '%{lit_distro_cfg}')" >> %{buildroot}%{lit_cfg}

# Install test script
install -d %{buildroot}%{_libexecdir}/tests/libomp
install -m 0755 %{SOURCE3} %{buildroot}%{_libexecdir}/tests/libomp

# Remove static libraries with equivalent shared libraries
rm -rf %{buildroot}%{_libdir}/libarcher_static.a

%check
%cmake_build --target check-openmp

%files
%license LICENSE.TXT
%{_libdir}/libomp.so
%{_libdir}/libompd.so
%{_libdir}/libarcher.so
%{_libdir}/libomptarget.rtl.amdgpu.so.%{maj_ver}
%{_libdir}/libomptarget.rtl.amdgpu.nextgen.so.%{maj_ver}
%{_libdir}/libomptarget.rtl.cuda.so.%{maj_ver}
%{_libdir}/libomptarget.rtl.cuda.nextgen.so.%{maj_ver}
%{_libdir}/libomptarget.rtl.%{libomp_arch}.so.%{maj_ver}
%{_libdir}/libomptarget.rtl.%{libomp_arch}.nextgen.so.%{maj_ver}
%{_libdir}/libomptarget.so.%{maj_ver}

%files devel
%{_libdir}/clang/%{maj_ver}/include/omp.h
%{_libdir}/cmake/openmp/FindOpenMPTarget.cmake
%{_libdir}/clang/%{maj_ver}/include/omp-tools.h
%{_libdir}/clang/%{maj_ver}/include/ompt.h
%{_libdir}/clang/%{maj_ver}/include/ompt-multiplex.h
%{_libdir}/libomptarget.rtl.amdgpu.so
%{_libdir}/libomptarget.rtl.amdgpu.nextgen.so
%{_libdir}/libomptarget.rtl.cuda.so
%{_libdir}/libomptarget.rtl.cuda.nextgen.so
%{_libdir}/libomptarget.rtl.%{libomp_arch}.so
%{_libdir}/libomptarget.rtl.%{libomp_arch}.nextgen.so
%{_libdir}/libomptarget.devicertl.a
%{_libdir}/libomptarget-amdgpu-*.bc
%{_libdir}/libomptarget-nvptx-*.bc
%{_libdir}/libomptarget.so

%files test
%{_datadir}/libomp
%{_libexecdir}/tests/libomp/

%changelog
* Tue Aug 8 2023 Funda Wang <fundawang@yeah.net> - 16.0.6-1
- New version 16.0.6

* Thu May 18 2023 Funda Wang <fundawang@yeah.net> - 16.0.4-1
- New version 16.0.4

* Fri Apr 14 2023 Funda Wang <fundawang@yeah.net> - 16.0.1-1
- New version 16.0.1

* Wed Feb 1 2023 Funda Wang <fundawang@yeah.net> - 15.0.7-1
- Import package for anolis 23
